@@ -33,44 +33,9 @@ public:
         : runtime_error(what_arg) { }
 };
 
-class ndJson
-{
-public:
-    ndJson();
-    ndJson(json_object *root);
-    void Destroy(void);
-
-    static json_object *CreateObject(void);
-    json_object *CreateObject(json_object *parent, const string &name);
-    json_object *CreateArray(json_object *parent, const string &name);
-
-    void AddObject(json_object *parent, const string &name, json_object *object);
-    void AddObject(json_object *parent, const string &name, const char *value);
-    void AddObject(json_object *parent, const string &name, const string &value);
-    void AddObject(json_object *parent, const string &name, int32_t value);
-    void AddObject(json_object *parent, const string &name, int64_t value);
-    void AddObject(json_object *parent, const string &name, uint32_t value);
-    void AddObject(json_object *parent, const string &name, uint64_t value);
-    void AddObject(json_object *parent, const string &name, double value);
-    void AddObject(json_object *parent, const string &name, bool value);
-
-    void PushObject(json_object *parent, const char *value);
-    void PushObject(json_object *parent, const string &value);
-    void PushObject(json_object *parent, int32_t value);
-    void PushObject(json_object *parent, int64_t value);
-    void PushObject(json_object *parent, double value);
-    void PushObject(json_object *parent, bool value);
-    void PushObject(json_object *parent, json_object *object);
-
-    void ToString(string &output, bool pretty = true);
-
-    void SaveToFile(const string &filename);
-
-    json_object *GetRoot(void) { return root; }
-
-protected:
-    json_object *root;
-};
+void nd_json_to_string(const json &j, string &output, bool pretty = false);
+void nd_json_save_to_file(const json &j, const string &filename, bool pretty = false);
+void nd_json_save_to_file(const string &j, const string &filename);
 
 typedef vector<string> ndJsonDataChunks;
 typedef map<string, ndJsonDataChunks> ndJsonData;
@@ -86,39 +51,27 @@ typedef map<string, ndJsonPluginParams> ndJsonPluginDispatch;
 class ndJsonObject
 {
 public:
-    ndJsonObject()
-    {
-        jtok = json_tokener_new_ex(ND_JSON_TOKENER_DEPTH);
-        if (jtok == NULL)
-            throw ndJsonInitException(strerror(ENOMEM));
-    }
+    ndJsonObject() {}
+    virtual ~ndJsonObject() {}
 
-    ndJsonObject(json_tokener *jtok)
-        : jtok(jtok) { }
-
-    virtual ~ndJsonObject()
-    {
-        if (jtok != NULL) json_tokener_free(jtok);
-    }
-
-    virtual void Parse(const string &json) = 0;
-
-protected:
-    json_tokener *jtok;
+    virtual void Parse(const string &json_string) = 0;
 };
 
 class ndJsonStatus : public ndJsonObject
 {
 public:
     ndJsonStatus()
-        : ndJsonObject(), timestamp(0), uptime(0), sink_queue_max_size_kb(0)
+        : ndJsonObject(), timestamp(0), uptime(0),
+        update_interval(0), update_imf(0),
+        sink_queue_max_size_kb(0)
     {
         memset(&stats, 0, sizeof(nd_agent_stats));
     }
 
-    virtual void Parse(const string &json);
+    virtual void Parse(const string &json_string);
 
     time_t timestamp, uptime;
+    unsigned update_interval, update_imf;
     uint32_t sink_queue_max_size_kb;
 
     nd_agent_stats stats;
@@ -132,10 +85,10 @@ public:
         update_imf(1), upload_enabled(false) { }
 
     ndJsonResponse(ndJsonResponseCode code, const string &message)
-        : ndJsonObject(NULL), version(0), resp_code(code), resp_message(message),
+        : ndJsonObject(), version(0), resp_code(code), resp_message(message),
         update_imf(1), upload_enabled(false) { }
 
-    virtual void Parse(const string &json);
+    virtual void Parse(const string &json_string);
 
     double version;
 
@@ -158,11 +111,11 @@ public:
 #endif
 
 protected:
-    void UnserializeData(json_object *jdata);
+    void UnserializeData(json &jdata);
 #ifdef _ND_USE_PLUGINS
-    void UnserializePluginRequest(json_object *jrequest, ndJsonPluginRequest &plugin_request);
-    void UnserializePluginDispatch(json_object *jdispatch);
-#endif
+    void UnserializePluginRequest(json &jrequest, ndJsonPluginRequest &plugin_request);
+    void UnserializePluginDispatch(json &jdispatch);
+#endif // _ND_USE_PLUGINS
 };
 
 #endif // _ND_JSON_H
