@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2018 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,7 +17,12 @@
 #ifndef _ND_UTIL_H
 #define _ND_UTIL_H
 
-#define ND_SHA1_BUFFER    4096
+#define ND_SHA1_BUFFER      4096
+
+#define ND_C_RESET          "\033[0m"
+#define ND_C_RED            "\033[0;31m"
+#define ND_C_GREEN          "\033[0;32m"
+#define ND_C_YELLOW         "\033[0;33m"
 
 void *nd_mem_alloc(size_t size);
 
@@ -46,9 +51,14 @@ void nd_iface_name(const string &iface, string &result);
 
 bool nd_is_ipaddr(const char *ip);
 
+void nd_private_ipaddr(uint8_t index, struct sockaddr_storage &addr);
+
 bool nd_load_uuid(string &uuid, const char *path, size_t length);
 bool nd_save_uuid(const string &uuid, const char *path, size_t length);
 
+bool nd_load_sink_url(string &url);
+bool nd_save_sink_url(const string &url);
+
 void nd_generate_uuid(string &uuid);
 
 string nd_get_version_and_features(void);
@@ -57,11 +67,32 @@ string nd_get_version_and_features(void);
 int nd_touch(const string &filename);
 #endif
 
+int nd_file_load(const string &filename, string &data);
+
 void nd_file_save(const string &filename, const string &data,
     bool append = false, mode_t mode = S_IRUSR | S_IWUSR | S_IRGRP,
     const char *user = NULL, const char *group = NULL);
 
 int nd_save_response_data(const char *filename, const ndJsonDataChunks &data);
 
+int nd_ifreq(const string &name, unsigned long request, struct ifreq *ifr);
+
+int nd_ifaddrs(nd_interface_addr_map &addr_map);
+int nd_ifaddrs_update(nd_interface_addr_map &addr_map);
+void nd_ifaddrs_free(nd_interface_addr_map &addr_map);
+bool nd_ifaddrs_get_mac(
+    nd_interface_addr_map &addr_map,
+    const string &name, uint8_t *addr);
+
+pid_t nd_is_running(pid_t pid, const char *exe_base);
+
+int nd_file_exists(const char *path);
+
+void nd_uptime(time_t ut, string &uptime);
+
+int nd_functions_exec(const string &func, string &output);
+
+void nd_os_detect(string &os);
+
 #endif // _ND_UTIL_H
 // vi: expandtab shiftwidth=4 softtabstop=4 tabstop=4
