@@ -1,7 +1,7 @@
 /*
  * ndpi_api.h
  *
- * Copyright (C) 2011-17 - ntop.org
+ * Copyright (C) 2011-19 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -21,7 +21,6 @@
  *
  */
 
-
 #ifndef __NDPI_API_H__
 #define __NDPI_API_H__
 
@@ -58,7 +57,7 @@ extern "C" {
      *         else 0
      *
      */
-    int check_punycode_string(char *buff, int len);
+    int ndpi_check_punycode_string(char *buff, int len);
 
 
     /**
@@ -226,16 +225,29 @@ extern "C" {
             const NDPI_PROTOCOL_BITMASK * detection_bitmask);
 
     /**
-     *  Function to be called before we give up with detection for a given flow.
-     *  This function reduces the NDPI_UNKNOWN_PROTOCOL detection
+     *  Function to be called to see in case of unknown match to see if there is
+     *  a partial match that has been prevented by the current nDPI preferences configuration
      *
      * @par    ndpi_struct  = the detection module
      * @par    flow         = the flow given for the detection module
      * @return the detected protocol even if the flow is not completed;
      *
      */
-    ndpi_protocol ndpi_detection_giveup(struct ndpi_detection_module_struct *ndpi_struct,
+    ndpi_protocol ndpi_get_partial_detection(struct ndpi_detection_module_struct *ndpi_struct,
             struct ndpi_flow_struct *flow);
+    /**
+     *  Function to be called before we give up with detection for a given flow.
+     *  This function reduces the NDPI_UNKNOWN_PROTOCOL detection
+     *
+     * @par    ndpi_struct  = the detection module
+     * @par    flow         = the flow given for the detection module
+     * @par    enable_guess = guess protocol if unknown
+     * @return the detected protocol even if the flow is not completed;
+     *
+     */
+    ndpi_protocol ndpi_detection_giveup(struct ndpi_detection_module_struct *ndpi_struct,
+            struct ndpi_flow_struct *flow,
+            u_int8_t enable_guess);
 
     /**
      * Processes an extra packet in order to get more information for a given protocol
@@ -344,6 +356,7 @@ extern "C" {
      * Search and return the protocol guessed that is undetected
      *
      * @par    ndpi_struct  = the detection module
+     * @par    flow         = the flow we're trying to guess, NULL if not available
      * @par    proto        = the l4 protocol number
      * @par    sport        = source port number
      * @par    dport        = destination port number
@@ -351,6 +364,7 @@ extern "C" {
      *
      */
     u_int16_t ndpi_guess_undetected_protocol(struct ndpi_detection_module_struct *ndpi_struct,
+            struct ndpi_flow_struct *flow,
             u_int8_t proto,
             u_int16_t sport,
             u_int16_t dport);
@@ -498,7 +512,7 @@ extern "C" {
      *
      */
     int ndpi_load_protocols_file(struct ndpi_detection_module_struct *ndpi_mod,
-            char* path);
+            const char* path);
 
     /**
      * Get the total number of the supported protocols
@@ -668,10 +682,16 @@ extern "C" {
 
     u_int ndpi_get_ndpi_num_custom_protocols(struct ndpi_detection_module_struct *ndpi_mod);
 
-    u_int ndpi_get_ndpi_detection_module_size();
+    u_int ndpi_get_ndpi_detection_module_size(void);
 
     void ndpi_set_log_level(struct ndpi_detection_module_struct *ndpi_mod, u_int l);
 
+    /* LRU cache */
+    struct ndpi_lru_cache* ndpi_lru_cache_init(u_int32_t num_entries);
+    void ndpi_lru_free_cache(struct ndpi_lru_cache *c);
+    u_int8_t ndpi_lru_find_cache(struct ndpi_lru_cache *c, u_int32_t key, u_int8_t clean_key_when_found);
+    void ndpi_lru_add_to_cache(struct ndpi_lru_cache *c, u_int32_t key);
+
     /**
      * Add a string to match to an automata
      *
@@ -690,12 +710,27 @@ extern "C" {
     void set_ndpi_flow_free(void  (*__ndpi_flow_free)(void *ptr));
     void set_ndpi_debug_function(struct ndpi_detection_module_struct *ndpi_str,
             ndpi_debug_function_ptr ndpi_debug_printf);
-    void * ndpi_malloc(size_t size);
-    void * ndpi_calloc(unsigned long count, size_t size);
-    void ndpi_free(void *ptr);
+    //void * ndpi_malloc(size_t size);
+    //void * ndpi_calloc(unsigned long count, size_t size);
+    //void ndpi_free(void *ptr);
     u_int8_t ndpi_get_api_version();
+
+    /* https://github.com/corelight/community-id-spec */
+    int ndpi_flowv4_flow_hash(u_int8_t l4_proto, u_int32_t src_ip, u_int32_t dst_ip, u_int16_t src_port, u_int16_t dst_port,
+            u_int8_t icmp_type, u_int8_t icmp_code, u_char *hash_buf, u_int8_t hash_buf_len);
+    int ndpi_flowv6_flow_hash(u_int8_t l4_proto, struct ndpi_in6_addr *src_ip, struct ndpi_in6_addr *dst_ip,
+            u_int16_t src_port, u_int16_t dst_port, u_int8_t icmp_type, u_int8_t icmp_code,
+            u_char *hash_buf, u_int8_t hash_buf_len);
+
+    u_int8_t ndpi_is_safe_ssl_cipher(u_int32_t cipher);
+    const char* ndpi_cipher2str(u_int32_t cipher);
+    u_int16_t ndpi_guess_host_protocol_id(struct ndpi_detection_module_struct *ndpi_struct,
+            struct ndpi_flow_struct *flow);
+    int ndpi_has_human_readeable_string(struct ndpi_detection_module_struct *ndpi_struct,
+            char *buffer, u_int buffer_size);
+    char* ndpi_ssl_version2str(u_int16_t version);
 #ifdef __cplusplus
 }
 #endif
 
-#endif	/* __NDPI_API_H__ */
+#endif  /* __NDPI_API_H__ */
