@@ -38,9 +38,11 @@
 #include <sys/stat.h>
 #include <sys/socket.h>
 
-#include <json.h>
 #include <pcap/pcap.h>
 
+#include <nlohmann/json.hpp>
+using json = nlohmann::json;
+
 using namespace std;
 
 #include "netifyd.h"
